-- Quartus II VHDL Template
-- Six-State Moore State Machine

-- A Moore machine's outputs are dependent only on the current state.
-- The output is written only when the state changes.  (State
-- transitions are synchronous.)

library ieee;
use ieee.std_logic_1164.all;

entity dado is
	generic	(
		tp_CC	: time := 1 ns;		-- para simulacao do tempo de atraso de um circuito combinacional
		tp_CS	: time := 2 ns		-- para simulacao do tempo de atraso de um circuito sequencial
	);
	port	(
		-- Sinais de controle
		clk		: in	std_logic;
		reset	: in	std_logic;
		-- Saidas de dados
		A, B, C, D	: out	std_logic
	);
end entity;

architecture behavioural of dado is
	-- Build an enumerated type for the state machine
	type state_type is (stateA, stateB, stateC, stateD, stateE, stateF);
	-- Register to hold the current state
	signal actual_state, next_state : state_type;
	signal output : std_logic_vector(3 downto 0);
begin
	--
	-- Bloco Sequencial (Elementos de memoria)
	--
	Memory: block
	begin
		-- Determina o proximo estado de forma sincrona
		process (clk, reset)
		begin
			if reset = '0' then
				actual_state <= stateA;
			elsif (rising_edge(clk)) then
				actual_state <= next_state after tp_CS;
			end if;
		end process;
	end block Memory;

	--
	-- Bloco Combinacional (Logica Combinacional)
	--
	Combinational_Logic: block
	begin
		-- Determina a saida e o proximo estado baseado apenas no
		-- estado atual (nao espera pela borda do clock)
		-- Valido para Modelo de Moore, apenas. No Modelo de Mealy, a
		-- saida é condicionada à entrada.
		process (actual_state)
		begin
			case actual_state is
			-- Main state machine
				when stateA=>
					--         ABCD
					output <= "1101" after tp_CC;
					next_state <= stateB;
				when stateB=>
					--         ABCD
					output <= "1011" after tp_CC;
					next_state <= stateC;
				when stateC=>
					--         ABCD
					output <= "1001" after tp_CC;
					next_state <= stateD;
				when stateD=>
					--         ABCD
					output <= "1010" after tp_CC;
					next_state <= stateE;
				when stateE=>
					--         ABCD
					output <= "1000" after tp_CC;
					next_state <= stateF;
				when stateF=>
					--         ABCD
					output <= "0010" after tp_CC;
					next_state <= stateA;
			end case;
		end process;
	end block Combinational_Logic;
	
	-- Gera as saidas para o painel de LEDS
	A <= output(3);
	B <= output(2);
	C <= output(1);
	D <= output(0);
end behavioural;
