-- Quartus II VHDL Template
-- Six-State Moore State Machine

-- A Moore machine's outputs are dependent only on the current state.
-- The output is written only when the state changes.  (State
-- transitions are synchronous.)

library ieee;
use ieee.std_logic_1164.all;

entity dado_PLD is
	generic	(
		tp_CC	: time := 1 ns;		-- para simulacao do tempo de atraso de um circuito combinacional
		tp_CS	: time := 2 ns		-- para simulacao do tempo de atraso de um circuito sequencial
	);
	port	(
		-- Sinais de controle
		clk		: in	std_logic;
		reset	: in	std_logic;
		-- Saidas de dados
		A, B, C, D	: out	std_logic
	);
end entity;

architecture behavioural of dado_PLD is
	-- Build an enumerated type for the state machine
	type state_type is (st0000, st0001, st0010, st0011, st0100, st0101, st0110, st0111, st1000, st1001, st1010, st1011, st1100, st1101, st1110, st1111);
	-- Register to hold the current state
	signal actual_state, next_state : state_type;
	signal output : std_logic_vector(3 downto 0);
begin
	--
	-- Bloco Sequencial (Elementos de memoria)
	--
	Memory: block
	begin
		-- Determina o proximo estado de forma sincrona
		process (clk, reset)
		begin
			if reset = '0' then
				actual_state <= st1101;
			elsif (rising_edge(clk)) then
				actual_state <= next_state after tp_CS;
			end if;
		end process;
	end block Memory;

	--
	-- Bloco Combinacional (Logica Combinacional)
	--
	Combinational_Logic: block
	begin
		-- Determina a saida e o proximo estado baseado apenas no
		-- estado atual (nao espera pela borda do clock)
		-- Valido para Modelo de Moore, apenas. No Modelo de Mealy, a
		-- saida é condicionada à entrada.
		process (actual_state)
		begin
			case actual_state is
			-- Main state machine
				when st1101=>
					--         ABCD
					output <= "1101" after tp_CC;
					next_state <= st1011;
				when st1011=>
					--         ABCD
					output <= "1011" after tp_CC;
					next_state <= st1001;
				when st1001=>
					--         ABCD
					output <= "1001" after tp_CC;
					next_state <= st1010;
				when st1010=>
					--         ABCD
					output <= "1010" after tp_CC;
					next_state <= st1000;
				when st1000=>
					--         ABCD
					output <= "1000" after tp_CC;
					next_state <= st0010;
				when st0010=>
					--         ABCD
					output <= "0010" after tp_CC;
					next_state <= st1101;
			-- Secundary state machine
			-- Proximo estado baseado nos Mapas de Karnaugh
			-- apresentados em sala de aula
				when st0000=>
					output <= "0000" after tp_CC;
					next_state <= st0111;
				when st0001=>
					output <= "0001" after tp_CC;
					next_state <= st1111;
				when st0011=>
					output <= "0011" after tp_CC;
					next_state <= st1101;
				when st0100=>
					output <= "0100" after tp_CC;
					next_state <= st0111;
				when st0101=>
					output <= "0101" after tp_CC;
					next_state <= st1111;
				when st0111=>
					output <= "0111" after tp_CC;
					next_state <= st1101;
				when st0110=>
					output <= "0110" after tp_CC;
					next_state <= st1101;
				when st1100=>
					output <= "1100" after tp_CC;
					next_state <= st0011;
				when st1111=>
					output <= "1111" after tp_CC;
					next_state <= st1001;
				when st1110=>
					output <= "1110" after tp_CC;
					next_state <= st1001;
			end case;
		end process;
	end block Combinational_Logic;

	-- Gera as saidas para o painel de LEDS
	A <= output(3);
	B <= output(2);
	C <= output(1);
	D <= output(0);
end behavioural;
