-- Quartus II VHDL Template
-- Four-State Mealy State Machine

-- A Mealy machine has outputs that depend on both the state and
-- the inputs.	When the inputs change, the outputs are updated
-- immediately, without waiting for a clock edge.  The outputs
-- can be written more than once per state or per clock cycle.

-- Implementação do Exemplo 8.6 do livro Digital Circuit Analysis & Design
--
--		      x
--		   0     1
--		-------------
--	A	| A/0 | B/0 |
--	B	| A/0 | C/1 |
--	C	| B/0 | D/0 |
--	D	| C/1 | D/0 |
--		-------------

library ieee;
use ieee.std_logic_1164.all;

entity fsm_8_6 is
	generic	(
		tp_CC	: time := 1 ns;		-- para simulacao do tempo de atraso de um circuito combinacional
		tp_CS	: time := 2 ns		-- para simulacao do tempo de atraso de um circuito sequencial
	);
	port	(
		-- Sinais de controle
		clk		: in	std_logic;	-- Clock
		reset	: in	std_logic;	-- Reset
		-- Entrada e saida de dados
		x		: in	std_logic;	-- Entrada
		z		: out	std_logic	-- Saida
	);
end entity;

architecture behavioural of fsm_8_6 is
	-- Build an enumerated type for the state machine
	type state_type is (stateA, stateB, stateC, stateD);
	-- Register to hold the current and next states
	signal actual_state, next_state : state_type;
begin
	--
	-- Bloco Sequencial (Memoria)
	--
	Memory: block
	begin
		-- Determina o proximo estado de forma sincrona
		process (clk, reset)
		begin
			-- Inicializacao da Maquina de Estados Finitos
			if reset = '0' then
				actual_state <= stateA;
			-- Atribuicao ao estado atual o proximo estado
			elsif (rising_edge(clk)) then
				actual_state <= next_state after tp_CS;
			end if;
		end process;
	end block Memory;

	--
	-- Bloco Combinacional (Logica Combinacional)
	--
	Combinational_Logic: block
	begin
		-- Determina a saida e o proximo estado baseado apenas no
		-- estado atual e na entrada (nao espera pela borda do clock)
		-- Valido para Modelo de Mealy, apenas. No Modelo de Moore, a
		-- saida é atribuida fora do bloco condicional.
		process (actual_state, x)
		begin
				case actual_state is
					when stateA=>
						if x = '0' then
							next_state <= stateA after tp_CC;
							z <= '0' after tp_CC;
						else
							next_state <= stateB after tp_CC;
							z <= '0' after tp_CC;
						end if;
					when stateB=>
						if x = '0' then
							next_state <= stateA after tp_CC;
							z <= '0' after tp_CC;
						else
							next_state <= stateC after tp_CC;
							z <= '1' after tp_CC;
						end if;
					when stateC=>
						if x = '0' then
							next_state <= stateB after tp_CC;
							z <= '0' after tp_CC;
						else
							next_state <= stateD after tp_CC;
							z <= '0' after tp_CC;
						end if;
					when stateD=>
						if x = '0' then
							next_state <= stateC after tp_CC;
							z <= '1' after tp_CC;
						else
							next_state <= stateD after tp_CC;
							z <= '0' after tp_CC;
						end if;
				end case;
		end process;
	end block Combinational_Logic;

end behavioural;
